@@ -38,11 +38,11 @@ import (
 	"strings"
 	"testing"
 
-	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/cmd"
+	"golang.org/x/tools/gopls/internal/debug"
 	"golang.org/x/tools/gopls/internal/hooks"
-	"golang.org/x/tools/gopls/internal/lsp/debug"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/internal/testenv"
 	"golang.org/x/tools/internal/tool"
 	"golang.org/x/tools/txtar"
@@ -205,14 +205,14 @@ func TestFail(t *testing.T) { t.Fatal("fail") }
 	{
 		res := gopls(t, tree, "codelens", "-exec", "./a/a_test.go:3", "run test")
 		res.checkExit(true)
-		res.checkStdout(`PASS: TestPass`)         // from go test
+		res.checkStderr(`PASS: TestPass`)         // from go test
 		res.checkStderr("Info: all tests passed") // from gopls.test
 	}
 	// run the failing test
 	{
 		res := gopls(t, tree, "codelens", "-exec", "./a/a_test.go:4", "run test")
 		res.checkExit(false)
-		res.checkStdout(`FAIL	example.com/a`)
+		res.checkStderr(`FAIL	example.com/a`)
 		res.checkStderr("Info: 1 / 1 tests failed")
 	}
 }
@@ -268,6 +268,80 @@ func g() {
 	}
 }
 
+// TestExecute tests the 'execute' subcommand (../execute.go).
+func TestExecute(t *testing.T) {
+	t.Parallel()
+
+	tree := writeTree(t, `
+-- go.mod --
+module example.com
+go 1.18
+
+-- hello.go --
+package a
+func main() {}
+
+-- hello_test.go --
+package a
+import "testing"
+func TestHello(t *testing.T) {
+	t.Fatal("oops")
+}
+`)
+	// missing command name
+	{
+		res := gopls(t, tree, "execute")
+		res.checkExit(false)
+		res.checkStderr("requires a command")
+	}
+	// bad command
+	{
+		res := gopls(t, tree, "execute", "gopls.foo")
+		res.checkExit(false)
+		res.checkStderr("unrecognized command: gopls.foo")
+	}
+	// too few arguments
+	{
+		res := gopls(t, tree, "execute", "gopls.run_tests")
+		res.checkExit(false)
+		res.checkStderr("expected 1 input arguments, got 0")
+	}
+	// too many arguments
+	{
+		res := gopls(t, tree, "execute", "gopls.run_tests", "null", "null")
+		res.checkExit(false)
+		res.checkStderr("expected 1 input arguments, got 2")
+	}
+	// argument is not JSON
+	{
+		res := gopls(t, tree, "execute", "gopls.run_tests", "hello")
+		res.checkExit(false)
+		res.checkStderr("argument 1 is not valid JSON: invalid character 'h'")
+	}
+	// add import, show diff
+	hello := "file://" + filepath.ToSlash(tree) + "/hello.go"
+	{
+		res := gopls(t, tree, "execute", "-d", "gopls.add_import", `{"ImportPath": "fmt", "URI": "`+hello+`"}`)
+		res.checkExit(true)
+		res.checkStdout(`[+]import "fmt"`)
+	}
+	// list known packages (has a result)
+	{
+		res := gopls(t, tree, "execute", "gopls.list_known_packages", `{"URI": "`+hello+`"}`)
+		res.checkExit(true)
+		res.checkStdout(`"fmt"`)
+		res.checkStdout(`"encoding/json"`)
+	}
+	// run tests
+	{
+		helloTest := "file://" + filepath.ToSlash(tree) + "/hello_test.go"
+		res := gopls(t, tree, "execute", "gopls.run_tests", `{"URI": "`+helloTest+`", "Tests": ["TestHello"]}`)
+		res.checkExit(false)
+		res.checkStderr(`hello_test.go:4: oops`)
+		res.checkStderr(`1 / 1 tests failed`)
+	}
+}
+
 // TestFoldingRanges tests the 'folding_ranges' subcommand (../folding_range.go).
 func TestFoldingRanges(t *testing.T) {
 	t.Parallel()
@@ -904,7 +978,7 @@ var _ io.Reader = C{}
 type C struct{}
 
 // Read implements io.Reader.
-func (C) Read(p []byte) (n int, err error) {
+func (c C) Read(p []byte) (n int, err error) {
 	panic("unimplemented")
 }
 `[1:]
@@ -968,7 +1042,7 @@ func writeTree(t *testing.T, archive string) string {
 	root := t.TempDir()
 
 	// This unfortunate step is required because gopls output
-	// expands symbolic links it its input file names (arguably it
+	// expands symbolic links in its input file names (arguably it
 	// should not), and on macOS the temp dir is in /var -> private/var.
 	root, err := filepath.EvalSymlinks(root)
 	if err != nil {
