@@ -203,7 +203,7 @@ func (s *Snapshot) resolveImportGraph() (*importGraph, error) {
 	s.mu.Unlock()
 
 	openPackages := make(map[PackageID]bool)
-	for _, fh := range s.overlays() {
+	for _, fh := range s.Overlays() {
 		mps, err := s.MetadataForFile(ctx, fh.URI())
 		if err != nil {
 			return nil, err
@@ -929,8 +929,7 @@ func (s *Snapshot) getPackageHandles(ctx context.Context, ids []PackageID) (map[
 // A packageHandleBuilder computes a batch of packageHandles concurrently,
 // sharing computed transitive reachability sets used to compute package keys.
 type packageHandleBuilder struct {
-	meta *metadata.Graph
-	s    *Snapshot
+	s *Snapshot
 
 	// nodes are assembled synchronously.
 	nodes map[typerefs.IndexID]*handleNode
