@@ -23,6 +23,14 @@ import (
 	"golang.org/x/tools/internal/memoize"
 )
 
+// A ParsedModule contains the results of parsing a go.mod file.
+type ParsedModule struct {
+	URI         protocol.DocumentURI
+	File        *modfile.File
+	Mapper      *protocol.Mapper
+	ParseErrors []*Diagnostic
+}
+
 // ParseMod parses a go.mod file, using a cache. It may return partial results and an error.
 func (s *Snapshot) ParseMod(ctx context.Context, fh file.Handle) (*ParsedModule, error) {
 	uri := fh.URI()
@@ -100,6 +108,14 @@ func parseModImpl(ctx context.Context, fh file.Handle) (*ParsedModule, error) {
 	}, parseErr
 }
 
+// A ParsedWorkFile contains the results of parsing a go.work file.
+type ParsedWorkFile struct {
+	URI         protocol.DocumentURI
+	File        *modfile.WorkFile
+	Mapper      *protocol.Mapper
+	ParseErrors []*Diagnostic
+}
+
 // ParseWork parses a go.work file, using a cache. It may return partial results and an error.
 // TODO(adonovan): move to new work.go file.
 func (s *Snapshot) ParseWork(ctx context.Context, fh file.Handle) (*ParsedWorkFile, error) {
@@ -187,7 +203,7 @@ func (s *Snapshot) goSum(ctx context.Context, modURI protocol.DocumentURI) []byt
 	// TODO(rfindley): but that's not right. Changes to sum files should
 	// invalidate content, even if it's nonexistent content.
 	sumURI := protocol.URIFromPath(sumFilename(modURI))
-	var sumFH file.Handle = s.FindFile(sumURI)
+	sumFH := s.FindFile(sumURI)
 	if sumFH == nil {
 		var err error
 		sumFH, err = s.view.fs.ReadFile(ctx, sumURI)
@@ -305,7 +321,7 @@ func (s *Snapshot) extractGoCommandErrors(ctx context.Context, goCmdError error)
 	_ = errors.As(goCmdError, &moduleErrs)
 
 	// Match the error against all the mod files in the workspace.
-	for _, uri := range s.ModFiles() {
+	for _, uri := range s.View().ModFiles() {
 		fh, err := s.ReadFile(ctx, uri)
 		if err != nil {
 			event.Error(ctx, "getting modfile for Go command error", err)
@@ -442,9 +458,7 @@ See https://github.com/golang/go/issues/39164 for more detail on this issue.`,
 
 	case strings.Contains(goCmdError, "updates to go.sum needed"), strings.Contains(goCmdError, "missing go.sum entry"):
 		var args []protocol.DocumentURI
-		for _, uri := range s.ModFiles() {
-			args = append(args, uri)
-		}
+		args = append(args, s.View().ModFiles()...)
 		tidyCmd, err := command.NewTidyCommand("Run go mod tidy", command.URIArgs{URIs: args})
 		if err != nil {
 			return nil, err
