@@ -6,19 +6,21 @@ package lsprpc
 
 import (
 	"context"
+	"encoding/json"
 	"errors"
 	"regexp"
 	"strings"
 	"testing"
 	"time"
 
+	"golang.org/x/tools/gopls/internal/debug"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
-	"golang.org/x/tools/gopls/internal/lsp/debug"
-	"golang.org/x/tools/gopls/internal/lsp/fake"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	"golang.org/x/tools/gopls/internal/test/integration/fake"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/jsonrpc2"
 	"golang.org/x/tools/internal/jsonrpc2/servertest"
+	"golang.org/x/tools/internal/testenv"
 )
 
 type FakeClient struct {
@@ -289,6 +291,8 @@ func (s *initServer) Initialize(ctx context.Context, params *protocol.ParamIniti
 }
 
 func TestEnvForwarding(t *testing.T) {
+	testenv.NeedsTool(t, "go")
+
 	ctx := context.Background()
 
 	server := &initServer{}
@@ -343,3 +347,30 @@ func TestListenParsing(t *testing.T) {
 		}
 	}
 }
+
+// For #59479, verify that empty slices are serialized as [].
+func TestEmptySlices(t *testing.T) {
+	// The LSP would prefer that empty slices be sent as [] rather than null.
+	const bad = `{"a":null}`
+	const good = `{"a":[]}`
+	var x struct {
+		A []string `json:"a"`
+	}
+	buf, _ := json.Marshal(x)
+	if string(buf) != bad {
+		// uninitialized is ezpected to give null
+		t.Errorf("unexpectedly got %s, want %s", buf, bad)
+	}
+	x.A = make([]string, 0)
+	buf, _ = json.Marshal(x)
+	if string(buf) != good {
+		// expect []
+		t.Errorf("unexpectedly got %s, want %s", buf, good)
+	}
+	x.A = []string{}
+	buf, _ = json.Marshal(x)
+	if string(buf) != good {
+		// expect []
+		t.Errorf("unexpectedly got %s, want %s", buf, good)
+	}
+}
