@@ -59,10 +59,13 @@ import (
 	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/go/types/objectpath"
 	"golang.org/x/tools/go/types/typeutil"
-	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/lsp/cache"
+	"golang.org/x/tools/gopls/internal/lsp/cache/metadata"
+	"golang.org/x/tools/gopls/internal/lsp/cache/parsego"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/safetoken"
+	"golang.org/x/tools/gopls/internal/util/bug"
+	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/typeparams"
@@ -73,9 +76,8 @@ import (
 // an object (or several coupled objects) within a single type-checked
 // syntax package.
 type renamer struct {
-	pkg                Package               // the syntax package in which the renaming is applied
+	pkg                *cache.Package        // the syntax package in which the renaming is applied
 	objsToUpdate       map[types.Object]bool // records progress of calls to check
-	hadConflicts       bool
 	conflicts          []string
 	from, to           string
 	satisfyConstraints map[satisfy.Constraint]bool
@@ -95,7 +97,7 @@ type PrepareItem struct {
 // The returned usererr is intended to be displayed to the user to explain why
 // the prepare fails. Probably we could eliminate the redundancy in returning
 // two errors, but for now this is done defensively.
-func PrepareRename(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position) (_ *PrepareItem, usererr, err error) {
+func PrepareRename(ctx context.Context, snapshot *cache.Snapshot, f file.Handle, pp protocol.Position) (_ *PrepareItem, usererr, err error) {
 	ctx, done := event.Start(ctx, "source.PrepareRename")
 	defer done()
 
@@ -149,7 +151,7 @@ func PrepareRename(ctx context.Context, snapshot Snapshot, f file.Handle, pp pro
 	}, nil, nil
 }
 
-func prepareRenamePackageName(ctx context.Context, snapshot Snapshot, pgf *ParsedGoFile) (*PrepareItem, error) {
+func prepareRenamePackageName(ctx context.Context, snapshot *cache.Snapshot, pgf *ParsedGoFile) (*PrepareItem, error) {
 	// Does the client support file renaming?
 	fileRenameSupported := false
 	for _, op := range snapshot.Options().SupportedResourceOperations {
@@ -213,7 +215,7 @@ func checkRenamable(obj types.Object) error {
 // Rename returns a map of TextEdits for each file modified when renaming a
 // given identifier within a package and a boolean value of true for renaming
 // package and false otherwise.
-func Rename(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]protocol.TextEdit, bool, error) {
+func Rename(ctx context.Context, snapshot *cache.Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]protocol.TextEdit, bool, error) {
 	ctx, done := event.Start(ctx, "source.Rename")
 	defer done()
 
@@ -287,7 +289,7 @@ func Rename(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.P
 }
 
 // renameOrdinary renames an ordinary (non-package) name throughout the workspace.
-func renameOrdinary(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
+func renameOrdinary(ctx context.Context, snapshot *cache.Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
 	// Type-check the referring package and locate the object(s).
 	//
 	// Unlike NarrowestPackageForFile, this operation prefers the
@@ -296,17 +298,17 @@ func renameOrdinary(ctx context.Context, snapshot Snapshot, f file.Handle, pp pr
 	// 'references' doesn't also want the widest variant: it
 	// computes the union across all variants.)
 	var targets map[types.Object]ast.Node
-	var pkg Package
+	var pkg *cache.Package
 	{
-		metas, err := snapshot.MetadataForFile(ctx, f.URI())
+		mps, err := snapshot.MetadataForFile(ctx, f.URI())
 		if err != nil {
 			return nil, err
 		}
-		RemoveIntermediateTestVariants(&metas)
-		if len(metas) == 0 {
+		metadata.RemoveIntermediateTestVariants(&mps)
+		if len(mps) == 0 {
 			return nil, fmt.Errorf("no package metadata for file %s", f.URI())
 		}
-		widest := metas[len(metas)-1] // widest variant may include _test.go files
+		widest := mps[len(mps)-1] // widest variant may include _test.go files
 		pkgs, err := snapshot.TypeCheck(ctx, widest.ID)
 		if err != nil {
 			return nil, err
@@ -351,7 +353,7 @@ func renameOrdinary(ctx context.Context, snapshot Snapshot, f file.Handle, pp pr
 		// of the type parameters, unlike methods).
 		switch obj.(type) { // avoid "obj :=" since cases reassign the var
 		case *types.TypeName:
-			if _, ok := obj.Type().(*typeparams.TypeParam); ok {
+			if _, ok := obj.Type().(*types.TypeParam); ok {
 				// As with capitalized function parameters below, type parameters are
 				// local.
 				goto skipObjectPath
@@ -476,14 +478,14 @@ func funcOrigin(fn *types.Func) *types.Func {
 // (This neglects obscure edge cases where a _test.go file changes the
 // selectors used only in an ITV, but life is short. Also sin must be
 // punished.)
-func typeCheckReverseDependencies(ctx context.Context, snapshot Snapshot, declURI protocol.DocumentURI, transitive bool) ([]Package, error) {
+func typeCheckReverseDependencies(ctx context.Context, snapshot *cache.Snapshot, declURI protocol.DocumentURI, transitive bool) ([]*cache.Package, error) {
 	variants, err := snapshot.MetadataForFile(ctx, declURI)
 	if err != nil {
 		return nil, err
 	}
 	// variants must include ITVs for the reverse dependency
 	// computation, but they are filtered out before we typecheck.
-	allRdeps := make(map[PackageID]*Metadata)
+	allRdeps := make(map[PackageID]*metadata.Package)
 	for _, variant := range variants {
 		rdeps, err := snapshot.ReverseDependencies(ctx, variant.ID, transitive)
 		if err != nil {
@@ -504,7 +506,7 @@ func typeCheckReverseDependencies(ctx context.Context, snapshot Snapshot, declUR
 
 	// Sort the packages into some topological order of the
 	// (unfiltered) metadata graph.
-	SortPostOrder(snapshot, ids)
+	metadata.SortPostOrder(snapshot, ids)
 
 	// Dependencies must be visited first since they can expand
 	// the search set. Ideally we would process the (filtered) set
@@ -520,36 +522,11 @@ func typeCheckReverseDependencies(ctx context.Context, snapshot Snapshot, declUR
 	return snapshot.TypeCheck(ctx, ids...)
 }
 
-// SortPostOrder sorts the IDs so that if x depends on y, then y appears before x.
-func SortPostOrder(meta MetadataSource, ids []PackageID) {
-	postorder := make(map[PackageID]int)
-	order := 0
-	var visit func(PackageID)
-	visit = func(id PackageID) {
-		if _, ok := postorder[id]; !ok {
-			postorder[id] = -1 // break recursion
-			if m := meta.Metadata(id); m != nil {
-				for _, depID := range m.DepsByPkgPath {
-					visit(depID)
-				}
-			}
-			order++
-			postorder[id] = order
-		}
-	}
-	for _, id := range ids {
-		visit(id)
-	}
-	sort.Slice(ids, func(i, j int) bool {
-		return postorder[ids[i]] < postorder[ids[j]]
-	})
-}
-
 // renameExported renames the object denoted by (pkgPath, objPath)
 // within the specified packages, along with any other objects that
 // must be renamed as a consequence. The slice of packages must be
 // topologically ordered.
-func renameExported(pkgs []Package, declPkgPath PackagePath, declObjPath objectpath.Path, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
+func renameExported(pkgs []*cache.Package, declPkgPath PackagePath, declObjPath objectpath.Path, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
 
 	// A target is a name for an object that is stable across types.Packages.
 	type target struct {
@@ -638,7 +615,7 @@ func renameExported(pkgs []Package, declPkgPath PackagePath, declObjPath objectp
 }
 
 // renamePackageName renames package declarations, imports, and go.mod files.
-func renamePackageName(ctx context.Context, s Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
+func renamePackageName(ctx context.Context, s *cache.Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
 	// Rename the package decl and all imports.
 	renamingEdits, err := renamePackage(ctx, s, f, newName)
 	if err != nil {
@@ -655,7 +632,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f file.Handle, newName P
 	// Get all workspace modules.
 	// TODO(adonovan): should this operate on all go.mod files,
 	// irrespective of whether they are included in the workspace?
-	modFiles := s.ModFiles()
+	modFiles := s.View().ModFiles()
 	for _, m := range modFiles {
 		fh, err := s.ReadFile(ctx, m)
 		if err != nil {
@@ -727,7 +704,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f file.Handle, newName P
 		}
 
 		// Calculate the edits to be made due to the change.
-		edits := s.Options().ComputeEdits(string(pm.Mapper.Content), string(newContent))
+		edits := diff.Bytes(pm.Mapper.Content, newContent)
 		renamingEdits[pm.URI] = append(renamingEdits[pm.URI], edits...)
 	}
 
@@ -741,7 +718,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f file.Handle, newName P
 // It updates package clauses and import paths for the renamed package as well
 // as any other packages affected by the directory renaming among all packages
 // known to the snapshot.
-func renamePackage(ctx context.Context, s Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
+func renamePackage(ctx context.Context, s *cache.Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
 	if strings.HasSuffix(string(newName), "_test") {
 		return nil, fmt.Errorf("cannot rename to _test package")
 	}
@@ -775,12 +752,12 @@ func renamePackage(ctx context.Context, s Snapshot, f file.Handle, newName Packa
 
 	// Rename package and import declarations in all relevant packages.
 	edits := make(map[protocol.DocumentURI][]diff.Edit)
-	for _, m := range allMetadata {
+	for _, mp := range allMetadata {
 		// Special case: x_test packages for the renamed package will not have the
 		// package path as a dir prefix, but still need their package clauses
 		// renamed.
-		if m.PkgPath == oldPkgPath+"_test" {
-			if err := renamePackageClause(ctx, m, s, newName+"_test", edits); err != nil {
+		if mp.PkgPath == oldPkgPath+"_test" {
+			if err := renamePackageClause(ctx, mp, s, newName+"_test", edits); err != nil {
 				return nil, err
 			}
 			continue
@@ -789,34 +766,34 @@ func renamePackage(ctx context.Context, s Snapshot, f file.Handle, newName Packa
 		// Subtle: check this condition before checking for valid module info
 		// below, because we should not fail this operation if unrelated packages
 		// lack module info.
-		if !strings.HasPrefix(string(m.PkgPath)+"/", string(oldPkgPath)+"/") {
+		if !strings.HasPrefix(string(mp.PkgPath)+"/", string(oldPkgPath)+"/") {
 			continue // not affected by the package renaming
 		}
 
-		if m.Module == nil {
+		if mp.Module == nil {
 			// This check will always fail under Bazel.
-			return nil, fmt.Errorf("cannot rename package: missing module information for package %q", m.PkgPath)
+			return nil, fmt.Errorf("cannot rename package: missing module information for package %q", mp.PkgPath)
 		}
 
-		if modulePath != PackagePath(m.Module.Path) {
+		if modulePath != PackagePath(mp.Module.Path) {
 			continue // don't edit imports if nested package and renaming package have different module paths
 		}
 
 		// Renaming a package consists of changing its import path and package name.
-		suffix := strings.TrimPrefix(string(m.PkgPath), string(oldPkgPath))
+		suffix := strings.TrimPrefix(string(mp.PkgPath), string(oldPkgPath))
 		newPath := newPathPrefix + suffix
 
-		pkgName := m.Name
-		if m.PkgPath == oldPkgPath {
+		pkgName := mp.Name
+		if mp.PkgPath == oldPkgPath {
 			pkgName = newName
 
-			if err := renamePackageClause(ctx, m, s, newName, edits); err != nil {
+			if err := renamePackageClause(ctx, mp, s, newName, edits); err != nil {
 				return nil, err
 			}
 		}
 
 		imp := ImportPath(newPath) // TODO(adonovan): what if newPath has vendor/ prefix?
-		if err := renameImports(ctx, s, m, imp, pkgName, edits); err != nil {
+		if err := renameImports(ctx, s, mp, imp, pkgName, edits); err != nil {
 			return nil, err
 		}
 	}
@@ -828,9 +805,9 @@ func renamePackage(ctx context.Context, s Snapshot, f file.Handle, newName Packa
 // the package described by the given metadata, to newName.
 //
 // Edits are written into the edits map.
-func renamePackageClause(ctx context.Context, m *Metadata, snapshot Snapshot, newName PackageName, edits map[protocol.DocumentURI][]diff.Edit) error {
+func renamePackageClause(ctx context.Context, mp *metadata.Package, snapshot *cache.Snapshot, newName PackageName, edits map[protocol.DocumentURI][]diff.Edit) error {
 	// Rename internal references to the package in the renaming package.
-	for _, uri := range m.CompiledGoFiles {
+	for _, uri := range mp.CompiledGoFiles {
 		fh, err := snapshot.ReadFile(ctx, uri)
 		if err != nil {
 			return err
@@ -858,8 +835,8 @@ func renamePackageClause(ctx context.Context, m *Metadata, snapshot Snapshot, ne
 // newPath and name newName.
 //
 // Edits are written into the edits map.
-func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath ImportPath, newName PackageName, allEdits map[protocol.DocumentURI][]diff.Edit) error {
-	rdeps, err := snapshot.ReverseDependencies(ctx, m.ID, false) // find direct importers
+func renameImports(ctx context.Context, snapshot *cache.Snapshot, mp *metadata.Package, newPath ImportPath, newName PackageName, allEdits map[protocol.DocumentURI][]diff.Edit) error {
+	rdeps, err := snapshot.ReverseDependencies(ctx, mp.ID, false) // find direct importers
 	if err != nil {
 		return err
 	}
@@ -884,7 +861,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 				continue // no package declaration
 			}
 			for _, imp := range f.File.Imports {
-				if rdep.DepsByImpPath[UnquoteImportPath(imp)] != m.ID {
+				if rdep.DepsByImpPath[metadata.UnquoteImportPath(imp)] != mp.ID {
 					continue // not the import we're looking for
 				}
 
@@ -912,7 +889,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 
 	// If the imported package's name hasn't changed,
 	// we don't need to rename references within each file.
-	if newName == m.Name {
+	if newName == mp.Name {
 		return nil
 	}
 
@@ -937,7 +914,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 				if imp.Name != nil {
 					continue // has explicit local name
 				}
-				if rdeps[id].DepsByImpPath[UnquoteImportPath(imp)] != m.ID {
+				if rdeps[id].DepsByImpPath[metadata.UnquoteImportPath(imp)] != mp.ID {
 					continue // not the import we're looking for
 				}
 
@@ -1004,7 +981,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 // consequence of the requested renamings.
 //
 // It returns an error if the renaming would cause a conflict.
-func renameObjects(newName string, pkg Package, targets ...types.Object) (map[protocol.DocumentURI][]diff.Edit, map[types.Object]bool, error) {
+func renameObjects(newName string, pkg *cache.Package, targets ...types.Object) (map[protocol.DocumentURI][]diff.Edit, map[types.Object]bool, error) {
 	r := renamer{
 		pkg:          pkg,
 		objsToUpdate: make(map[types.Object]bool),
@@ -1089,7 +1066,7 @@ func (r *renamer) update() (map[protocol.DocumentURI][]diff.Edit, error) {
 		return items[i].node.Pos() < items[j].node.Pos()
 	})
 
-	// Update each identifier.
+	// Update each identifier, and its doc comment if it is a declaration.
 	for _, item := range items {
 		pgf, ok := enclosingFile(r.pkg, item.node.Pos())
 		if !ok {
@@ -1164,9 +1141,214 @@ func (r *renamer) update() (map[protocol.DocumentURI][]diff.Edit, error) {
 		}
 	}
 
+	docLinkEdits, err := r.updateCommentDocLinks()
+	if err != nil {
+		return nil, err
+	}
+	for uri, edits := range docLinkEdits {
+		result[uri] = append(result[uri], edits...)
+	}
+
 	return result, nil
 }
 
+// updateCommentDocLinks updates each doc comment in the package
+// that refers to one of the renamed objects using a doc link
+// (https://golang.org/doc/comment#doclinks) such as "[pkg.Type.Method]".
+func (r *renamer) updateCommentDocLinks() (map[protocol.DocumentURI][]diff.Edit, error) {
+	result := make(map[protocol.DocumentURI][]diff.Edit)
+	var docRenamers []*docLinkRenamer
+	for obj := range r.objsToUpdate {
+		if _, ok := obj.(*types.PkgName); ok {
+			// The dot package name will not be referenced
+			if obj.Name() == "." {
+				continue
+			}
+
+			docRenamers = append(docRenamers, &docLinkRenamer{
+				isDep:       false,
+				isPkgOrType: true,
+				file:        r.pkg.FileSet().File(obj.Pos()),
+				regexp:      docLinkPattern("", "", obj.Name(), true),
+				to:          r.to,
+			})
+			continue
+		}
+		if !obj.Exported() {
+			continue
+		}
+		recvName := ""
+		// Doc links can reference only exported package-level objects
+		// and methods of exported package-level named types.
+		if !isPackageLevel(obj) {
+			_, isFunc := obj.(*types.Func)
+			if !isFunc {
+				continue
+			}
+			recv := obj.Type().(*types.Signature).Recv()
+			if recv == nil {
+				continue
+			}
+			recvT := recv.Type()
+			if ptr, ok := recvT.(*types.Pointer); ok {
+				recvT = ptr.Elem()
+			}
+			named, isNamed := recvT.(*types.Named)
+			if !isNamed {
+				continue
+			}
+			// Doc links can't reference interface methods.
+			if types.IsInterface(named.Underlying()) {
+				continue
+			}
+			name := named.Origin().Obj()
+			if !name.Exported() || !isPackageLevel(name) {
+				continue
+			}
+			recvName = name.Name()
+		}
+
+		// Qualify objects from other packages.
+		pkgName := ""
+		if r.pkg.GetTypes() != obj.Pkg() {
+			pkgName = obj.Pkg().Name()
+		}
+		_, isTypeName := obj.(*types.TypeName)
+		docRenamers = append(docRenamers, &docLinkRenamer{
+			isDep:       r.pkg.GetTypes() != obj.Pkg(),
+			isPkgOrType: isTypeName,
+			packagePath: obj.Pkg().Path(),
+			packageName: pkgName,
+			recvName:    recvName,
+			objName:     obj.Name(),
+			regexp:      docLinkPattern(pkgName, recvName, obj.Name(), isTypeName),
+			to:          r.to,
+		})
+	}
+	for _, pgf := range r.pkg.CompiledGoFiles() {
+		for _, d := range docRenamers {
+			edits, err := d.update(pgf)
+			if err != nil {
+				return nil, err
+			}
+			if len(edits) > 0 {
+				result[pgf.URI] = append(result[pgf.URI], edits...)
+			}
+		}
+	}
+	return result, nil
+}
+
+// docLinkPattern returns a regular expression that matches doclinks in comments.
+// It has one submatch that indicates the symbol to be updated.
+func docLinkPattern(pkgName, recvName, objName string, isPkgOrType bool) *regexp.Regexp {
+	// The doc link may contain a leading star, e.g. [*bytes.Buffer].
+	pattern := `\[\*?`
+	if pkgName != "" {
+		pattern += pkgName + `\.`
+	}
+	if recvName != "" {
+		pattern += recvName + `\.`
+	}
+	// The first submatch is object name.
+	pattern += `(` + objName + `)`
+	// If the object is a *types.TypeName or *types.PkgName, also need
+	// match the objects referenced by them, so add `(\.\w+)*`.
+	if isPkgOrType {
+		pattern += `(?:\.\w+)*`
+	}
+	// There are two type of link in comments:
+	//   1. url link. e.g. [text]: url
+	//   2. doc link. e.g. [pkg.Name]
+	// in order to only match the doc link, add `([^:]|$)` in the end.
+	pattern += `\](?:[^:]|$)`
+
+	return regexp.MustCompile(pattern)
+}
+
+// A docLinkRenamer renames doc links of forms such as these:
+//
+//	[Func]
+//	[pkg.Func]
+//	[RecvType.Method]
+//	[*Type]
+//	[*pkg.Type]
+//	[*pkg.RecvType.Method]
+type docLinkRenamer struct {
+	isDep       bool // object is from a dependency package
+	isPkgOrType bool // object is *types.PkgName or *types.TypeName
+	packagePath string
+	packageName string // e.g. "pkg"
+	recvName    string // e.g. "RecvType"
+	objName     string // e.g. "Func", "Type", "Method"
+	to          string // new name
+	regexp      *regexp.Regexp
+
+	file *token.File // enclosing file, if renaming *types.PkgName
+}
+
+// update updates doc links in the package level comments.
+func (r *docLinkRenamer) update(pgf *parsego.File) (result []diff.Edit, err error) {
+	if r.file != nil && r.file != pgf.Tok {
+		return nil, nil
+	}
+	pattern := r.regexp
+	// If the object is in dependency package,
+	// the imported name in the file may be different from the original package name
+	if r.isDep {
+		for _, spec := range pgf.File.Imports {
+			importPath, _ := strconv.Unquote(spec.Path.Value)
+			if importPath == r.packagePath {
+				// Ignore blank imports
+				if spec.Name == nil || spec.Name.Name == "_" || spec.Name.Name == "." {
+					continue
+				}
+				if spec.Name.Name != r.packageName {
+					pattern = docLinkPattern(spec.Name.Name, r.recvName, r.objName, r.isPkgOrType)
+				}
+				break
+			}
+		}
+	}
+
+	var edits []diff.Edit
+	updateDocLinks := func(doc *ast.CommentGroup) error {
+		if doc != nil {
+			for _, c := range doc.List {
+				for _, locs := range pattern.FindAllStringSubmatchIndex(c.Text, -1) {
+					// The first submatch is the object name, so the locs[2:4] is the index of object name.
+					edit, err := posEdit(pgf.Tok, c.Pos()+token.Pos(locs[2]), c.Pos()+token.Pos(locs[3]), r.to)
+					if err != nil {
+						return err
+					}
+					edits = append(edits, edit)
+				}
+			}
+		}
+		return nil
+	}
+
+	// Update package doc comments.
+	err = updateDocLinks(pgf.File.Doc)
+	if err != nil {
+		return nil, err
+	}
+	for _, decl := range pgf.File.Decls {
+		var doc *ast.CommentGroup
+		switch decl := decl.(type) {
+		case *ast.GenDecl:
+			doc = decl.Doc
+		case *ast.FuncDecl:
+			doc = decl.Doc
+		}
+		err = updateDocLinks(doc)
+		if err != nil {
+			return nil, err
+		}
+	}
+	return edits, nil
+}
+
 // docComment returns the doc for an identifier within the specified file.
 func docComment(pgf *ParsedGoFile, id *ast.Ident) *ast.CommentGroup {
 	nodes, _ := astutil.PathEnclosingInterval(pgf.File, id.Pos(), id.End())
@@ -1246,7 +1428,7 @@ func (r *renamer) updatePkgName(pgf *ParsedGoFile, pkgName *types.PkgName) (diff
 // whether the position ppos lies within it.
 //
 // Note: also used by references.
-func parsePackageNameDecl(ctx context.Context, snapshot Snapshot, fh file.Handle, ppos protocol.Position) (*ParsedGoFile, bool, error) {
+func parsePackageNameDecl(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, ppos protocol.Position) (*ParsedGoFile, bool, error) {
 	pgf, err := snapshot.ParseGo(ctx, fh, ParseHeader)
 	if err != nil {
 		return nil, false, err
@@ -1258,7 +1440,7 @@ func parsePackageNameDecl(ctx context.Context, snapshot Snapshot, fh file.Handle
 }
 
 // enclosingFile returns the CompiledGoFile of pkg that contains the specified position.
-func enclosingFile(pkg Package, pos token.Pos) (*ParsedGoFile, bool) {
+func enclosingFile(pkg *cache.Package, pos token.Pos) (*ParsedGoFile, bool) {
 	for _, pgf := range pkg.CompiledGoFiles() {
 		if pgf.File.Pos() <= pos && pos <= pgf.File.End() {
 			return pgf, true
