@@ -34,8 +34,10 @@ import (
 	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/gopls/internal/telemetry"
 	"golang.org/x/tools/gopls/internal/util/bug"
+	"golang.org/x/tools/gopls/internal/util/maps"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/gopls/internal/vulncheck/scan"
+	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/tokeninternal"
@@ -140,37 +142,38 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 		}
 	}
 	var deps commandDeps
+	var release func()
 	if cfg.forURI != "" && cfg.forView != "" {
 		return bug.Errorf("internal error: forURI=%q, forView=%q", cfg.forURI, cfg.forView)
 	}
 	if cfg.forURI != "" {
-		var ok bool
-		var release func()
-		deps.snapshot, deps.fh, ok, release, err = c.s.beginFileRequest(ctx, cfg.forURI, file.UnknownKind)
-		defer release()
-		if !ok {
-			if err != nil {
-				return err
-			}
-			return fmt.Errorf("invalid file URL: %v", cfg.forURI)
+		deps.fh, deps.snapshot, release, err = c.s.fileOf(ctx, cfg.forURI)
+		if err != nil {
+			return err
 		}
+
 	} else if cfg.forView != "" {
 		view, err := c.s.session.View(cfg.forView)
 		if err != nil {
 			return err
 		}
-		var release func()
 		deps.snapshot, release, err = view.Snapshot()
 		if err != nil {
 			return err
 		}
-		defer release()
+
+	} else {
+		release = func() {}
 	}
+	// Inv: release() must be called exactly once after this point.
+	// In the async case, runcmd may outlive run().
+
 	ctx, cancel := context.WithCancel(xcontext.Detach(ctx))
 	if cfg.progress != "" {
 		deps.work = c.s.progress.Start(ctx, cfg.progress, "Running...", c.params.WorkDoneToken, cancel)
 	}
 	runcmd := func() error {
+		defer release()
 		defer cancel()
 		err := run(ctx, deps)
 		if deps.work != nil {
@@ -282,10 +285,9 @@ func (c *commandHandler) CheckUpgrades(ctx context.Context, args command.CheckUp
 			if err != nil {
 				return nil, nil, err
 			}
-			snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+			return c.s.session.InvalidateView(ctx, deps.snapshot.View(), cache.StateChange{
 				ModuleUpgrades: map[protocol.DocumentURI]map[string]string{args.URI: upgrades},
 			})
-			return snapshot, release, nil
 		})
 	})
 }
@@ -303,7 +305,7 @@ func (c *commandHandler) ResetGoModDiagnostics(ctx context.Context, args command
 		forURI: args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
 		return c.modifyState(ctx, FromResetGoModDiagnostics, func() (*cache.Snapshot, func(), error) {
-			snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+			return c.s.session.InvalidateView(ctx, deps.snapshot.View(), cache.StateChange{
 				ModuleUpgrades: map[protocol.DocumentURI]map[string]string{
 					deps.fh.URI(): nil,
 				},
@@ -311,7 +313,6 @@ func (c *commandHandler) ResetGoModDiagnostics(ctx context.Context, args command
 					deps.fh.URI(): nil,
 				},
 			})
-			return snapshot, release, nil
 		})
 	})
 }
@@ -333,11 +334,11 @@ func (c *commandHandler) UpdateGoSum(ctx context.Context, args command.URIArgs)
 		progress: "Updating go.sum",
 	}, func(ctx context.Context, _ commandDeps) error {
 		for _, uri := range args.URIs {
-			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, file.UnknownKind)
-			defer release()
-			if !ok {
+			fh, snapshot, release, err := c.s.fileOf(ctx, uri)
+			if err != nil {
 				return err
 			}
+			defer release()
 			if err := c.s.runGoModUpdateCommands(ctx, snapshot, fh.URI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
 				_, err := invoke("list", "all")
 				return err
@@ -355,11 +356,11 @@ func (c *commandHandler) Tidy(ctx context.Context, args command.URIArgs) error {
 		progress:    "Running go mod tidy",
 	}, func(ctx context.Context, _ commandDeps) error {
 		for _, uri := range args.URIs {
-			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, file.UnknownKind)
-			defer release()
-			if !ok {
+			fh, snapshot, release, err := c.s.fileOf(ctx, uri)
+			if err != nil {
 				return err
 			}
+			defer release()
 			if err := c.s.runGoModUpdateCommands(ctx, snapshot, fh.URI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
 				_, err := invoke("mod", "tidy")
 				return err
@@ -385,12 +386,16 @@ func (c *commandHandler) Vendor(ctx context.Context, args command.URIArg) error
 		// If golang/go#44119 is resolved, go mod vendor will instead modify
 		// modules.txt in-place. In that case we could theoretically allow this
 		// command to run concurrently.
+		stderr := new(bytes.Buffer)
 		err := deps.snapshot.RunGoCommandPiped(ctx, cache.Normal|cache.AllowNetwork, &gocommand.Invocation{
 			Verb:       "mod",
 			Args:       []string{"vendor"},
 			WorkingDir: filepath.Dir(args.URI.Path()),
-		}, &bytes.Buffer{}, &bytes.Buffer{})
-		return err
+		}, &bytes.Buffer{}, stderr)
+		if err != nil {
+			return fmt.Errorf("running go mod vendor failed: %v\nstderr:\n%s", err, stderr.String())
+		}
+		return nil
 	})
 }
 
@@ -399,11 +404,11 @@ func (c *commandHandler) EditGoDirective(ctx context.Context, args command.EditG
 		requireSave: true, // if go.mod isn't saved it could cause a problem
 		forURI:      args.URI,
 	}, func(ctx context.Context, _ commandDeps) error {
-		snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, args.URI, file.UnknownKind)
-		defer release()
-		if !ok {
+		fh, snapshot, release, err := c.s.fileOf(ctx, args.URI)
+		if err != nil {
 			return err
 		}
+		defer release()
 		if err := c.s.runGoModUpdateCommands(ctx, snapshot, fh.URI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
 			_, err := invoke("mod", "edit", "-go", args.Version)
 			return err
@@ -436,7 +441,7 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 		if err != nil {
 			return err
 		}
-		edits, err := dropDependency(deps.snapshot, pm, args.ModulePath)
+		edits, err := dropDependency(pm, args.ModulePath)
 		if err != nil {
 			return err
 		}
@@ -469,7 +474,7 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 
 // dropDependency returns the edits to remove the given require from the go.mod
 // file.
-func dropDependency(snapshot *cache.Snapshot, pm *cache.ParsedModule, modulePath string) ([]protocol.TextEdit, error) {
+func dropDependency(pm *cache.ParsedModule, modulePath string) ([]protocol.TextEdit, error) {
 	// We need a private copy of the parsed go.mod file, since we're going to
 	// modify it.
 	copied, err := modfile.Parse("", pm.Mapper.Content, nil)
@@ -485,7 +490,7 @@ func dropDependency(snapshot *cache.Snapshot, pm *cache.ParsedModule, modulePath
 		return nil, err
 	}
 	// Calculate the edits to be made due to the change.
-	diff := snapshot.Options().ComputeEdits(string(pm.Mapper.Content), string(newContent))
+	diff := diff.Bytes(pm.Mapper.Content, newContent)
 	return protocol.EditsFromDiffEdits(pm.Mapper, diff)
 }
 
@@ -688,7 +693,7 @@ func collectFileEdits(ctx context.Context, snapshot *cache.Snapshot, uri protoco
 	}
 
 	m := protocol.NewMapper(fh.URI(), oldContent)
-	diff := snapshot.Options().ComputeEdits(string(oldContent), string(newContent))
+	diff := diff.Bytes(oldContent, newContent)
 	edits, err := protocol.EditsFromDiffEdits(m, diff)
 	if err != nil {
 		return nil, err
@@ -789,12 +794,11 @@ func (c *commandHandler) ToggleGCDetails(ctx context.Context, args command.URIAr
 				return nil, nil, err
 			}
 			wantDetails := !deps.snapshot.WantGCDetails(meta.ID) // toggle the gc details state
-			snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+			return c.s.session.InvalidateView(ctx, deps.snapshot.View(), cache.StateChange{
 				GCDetails: map[metadata.PackageID]bool{
 					meta.ID: wantDetails,
 				},
 			})
-			return snapshot, release, nil
 		})
 	})
 }
@@ -939,22 +943,11 @@ func (c *commandHandler) StopProfile(ctx context.Context, args command.StopProfi
 	return result, nil
 }
 
-// Copy of pkgLoadConfig defined in internal/cmd/vulncheck.go
-// TODO(hyangah): decide where to define this.
-type pkgLoadConfig struct {
-	// BuildFlags is a list of command-line flags to be passed through to
-	// the build system's query tool.
-	BuildFlags []string
-
-	// If Tests is set, the loader includes related test packages.
-	Tests bool
-}
-
 func (c *commandHandler) FetchVulncheckResult(ctx context.Context, arg command.URIArg) (map[protocol.DocumentURI]*vulncheck.Result, error) {
 	ret := map[protocol.DocumentURI]*vulncheck.Result{}
 	err := c.run(ctx, commandConfig{forURI: arg.URI}, func(ctx context.Context, deps commandDeps) error {
 		if deps.snapshot.Options().Vulncheck == settings.ModeVulncheckImports {
-			for _, modfile := range deps.snapshot.ModFiles() {
+			for _, modfile := range deps.snapshot.View().ModFiles() {
 				res, err := deps.snapshot.ModVuln(ctx, modfile)
 				if err != nil {
 					return err
@@ -999,9 +992,12 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 			return err
 		}
 
-		snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+		snapshot, release, err := c.s.session.InvalidateView(ctx, deps.snapshot.View(), cache.StateChange{
 			Vulns: map[protocol.DocumentURI]*vulncheck.Result{args.URI: result},
 		})
+		if err != nil {
+			return err
+		}
 		defer release()
 		c.s.diagnoseSnapshot(snapshot, nil, 0)
 
@@ -1164,21 +1160,23 @@ func (c *commandHandler) RunGoWorkCommand(ctx context.Context, args command.RunG
 		view := snapshot.View()
 		viewDir := snapshot.Folder().Path()
 
-		// If the user has explicitly set GOWORK=off, we should warn them
-		// explicitly and avoid potentially misleading errors below.
-		goworkURI, off := view.GOWORK()
-		if off {
+		if view.Type() != cache.GoWorkView && view.GoWork() != "" {
+			// If we are not using an existing go.work file, GOWORK must be explicitly off.
+			// TODO(rfindley): what about GO111MODULE=off?
 			return fmt.Errorf("cannot modify go.work files when GOWORK=off")
 		}
-		gowork := goworkURI.Path()
 
-		if goworkURI != "" {
-			fh, err := snapshot.ReadFile(ctx, goworkURI)
+		var gowork string
+		// If the user has explicitly set GOWORK=off, we should warn them
+		// explicitly and avoid potentially misleading errors below.
+		if view.GoWork() != "" {
+			gowork = view.GoWork().Path()
+			fh, err := snapshot.ReadFile(ctx, view.GoWork())
 			if err != nil {
-				return fmt.Errorf("reading current go.work file: %v", err)
+				return err // e.g. canceled
 			}
 			if !fh.SameContentsOnDisk() {
-				return fmt.Errorf("must save workspace file %s before running go work commands", goworkURI)
+				return fmt.Errorf("must save workspace file %s before running go work commands", view.GoWork())
 			}
 		} else {
 			if !args.InitFirst {
@@ -1294,7 +1292,7 @@ func (c *commandHandler) ChangeSignature(ctx context.Context, args command.Chang
 func (c *commandHandler) DiagnoseFiles(ctx context.Context, args command.DiagnoseFilesArgs) error {
 	return c.run(ctx, commandConfig{
 		progress: "Diagnose files",
-	}, func(ctx context.Context, deps commandDeps) error {
+	}, func(ctx context.Context, _ commandDeps) error {
 
 		// TODO(rfindley): even better would be textDocument/diagnostics (golang/go#60122).
 		// Though note that implementing pull diagnostics may cause some servers to
@@ -1307,12 +1305,14 @@ func (c *commandHandler) DiagnoseFiles(ctx context.Context, args command.Diagnos
 		// grouping file URIs by package and making a
 		// single call to source.Analyze.
 		for _, uri := range args.Files {
-			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, file.UnknownKind)
-			defer release()
-			if !ok {
+			fh, snapshot, release, err := c.s.fileOf(ctx, uri)
+			if err != nil {
 				return err
 			}
-
+			defer release()
+			if snapshot.FileKind(fh) != file.Go {
+				continue
+			}
 			pkg, _, err := source.NarrowestPackageForFile(ctx, snapshot, uri)
 			if err != nil {
 				return err
@@ -1329,8 +1329,9 @@ func (c *commandHandler) DiagnoseFiles(ctx context.Context, args command.Diagnos
 			// combine load/parse/type + analysis diagnostics
 			var td, ad []*cache.Diagnostic
 			combineDiagnostics(pkgDiags, adiags[uri], &td, &ad)
-			c.s.storeDiagnostics(snapshot, uri, typeCheckSource, td)
-			c.s.storeDiagnostics(snapshot, uri, analysisSource, ad)
+			diags := append(td, ad...)
+			byURI := func(d *cache.Diagnostic) protocol.DocumentURI { return d.URI }
+			c.s.updateDiagnostics(ctx, c.s.session.Views(), snapshot, maps.Group(diags, byURI), false)
 			diagnostics := append(td, ad...)
 
 			if err := c.s.client.PublishDiagnostics(ctx, &protocol.PublishDiagnosticsParams{
@@ -1344,3 +1345,16 @@ func (c *commandHandler) DiagnoseFiles(ctx context.Context, args command.Diagnos
 		return nil
 	})
 }
+
+func (c *commandHandler) Views(ctx context.Context) ([]command.View, error) {
+	var summaries []command.View
+	for _, view := range c.s.session.Views() {
+		summaries = append(summaries, command.View{
+			Type:       view.Type().String(),
+			Root:       view.Root(),
+			Folder:     view.Folder().Dir,
+			EnvOverlay: view.EnvOverlay(),
+		})
+	}
+	return summaries, nil
+}
