@@ -51,25 +51,24 @@ import (
 	"golang.org/x/tools/go/analysis/passes/unsafeptr"
 	"golang.org/x/tools/go/analysis/passes/unusedresult"
 	"golang.org/x/tools/go/analysis/passes/unusedwrite"
+	"golang.org/x/tools/gopls/internal/analysis/deprecated"
+	"golang.org/x/tools/gopls/internal/analysis/embeddirective"
+	"golang.org/x/tools/gopls/internal/analysis/fillreturns"
+	"golang.org/x/tools/gopls/internal/analysis/fillstruct"
+	"golang.org/x/tools/gopls/internal/analysis/infertypeargs"
+	"golang.org/x/tools/gopls/internal/analysis/nonewvars"
+	"golang.org/x/tools/gopls/internal/analysis/noresultvalues"
+	"golang.org/x/tools/gopls/internal/analysis/simplifycompositelit"
+	"golang.org/x/tools/gopls/internal/analysis/simplifyrange"
+	"golang.org/x/tools/gopls/internal/analysis/simplifyslice"
+	"golang.org/x/tools/gopls/internal/analysis/stubmethods"
+	"golang.org/x/tools/gopls/internal/analysis/undeclaredname"
+	"golang.org/x/tools/gopls/internal/analysis/unusedparams"
+	"golang.org/x/tools/gopls/internal/analysis/unusedvariable"
+	"golang.org/x/tools/gopls/internal/analysis/useany"
 	"golang.org/x/tools/gopls/internal/file"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/deprecated"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/embeddirective"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/fillreturns"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/fillstruct"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/infertypeargs"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/nonewvars"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/noresultvalues"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/simplifycompositelit"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/simplifyrange"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/simplifyslice"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/stubmethods"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/undeclaredname"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/unusedparams"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/unusedvariable"
-	"golang.org/x/tools/gopls/internal/lsp/analysis/useany"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/internal/diff"
 )
 
 type Annotation string
@@ -117,7 +116,7 @@ func (opts *Options) IsAnalyzerEnabled(name string) bool {
 // ClientOptions holds LSP-specific configuration that is provided by the
 // client.
 type ClientOptions struct {
-	ClientInfo                                 *protocol.Msg_XInitializeParams_clientInfo
+	ClientInfo                                 *protocol.ClientInfo
 	InsertTextFormat                           protocol.InsertTextFormat
 	ConfigurationSupported                     bool
 	DynamicConfigurationSupported              bool
@@ -175,18 +174,17 @@ type BuildOptions struct {
 	// is the part of the file name after the final dot.)
 	TemplateExtensions []string
 
-	// MemoryMode controls the tradeoff `gopls` makes between memory usage and
-	// correctness.
-	//
-	// Values other than `Normal` are untested and may break in surprising ways.
-	MemoryMode MemoryMode `status:"experimental"`
+	// obsolete, no effect
+	MemoryMode string `status:"experimental"`
 
-	// ExpandWorkspaceToModule instructs `gopls` to adjust the scope of the
-	// workspace to find the best available module root. `gopls` first looks for
-	// a go.mod file in any parent directory of the workspace folder, expanding
-	// the scope to that directory if it exists. If no viable parent directory is
-	// found, gopls will check if there is exactly one child directory containing
-	// a go.mod file, narrowing the scope to that directory if it exists.
+	// ExpandWorkspaceToModule determines which packages are considered
+	// "workspace packages" when the workspace is using modules.
+	//
+	// Workspace packages affect the scope of workspace-wide operations. Notably,
+	// gopls diagnoses all packages considered to be part of the workspace after
+	// every keystroke, so by setting "ExpandWorkspaceToModule" to false, and
+	// opening a nested workspace directory, you can reduce the amount of work
+	// gopls has to do to keep your workspace up to date.
 	ExpandWorkspaceToModule bool `status:"experimental"`
 
 	// AllowModfileModifications disables -mod=readonly, allowing imports from
@@ -437,25 +435,15 @@ func (u *UserOptions) SetEnvSlice(env []string) {
 	}
 }
 
-// DiffFunction is the type for a function that produces a set of edits that
-// convert from the before content to the after content.
-type DiffFunction func(before, after string) []diff.Edit
-
 // Hooks contains configuration that is provided to the Gopls command by the
 // main package.
 type Hooks struct {
 	// LicensesText holds third party licenses for software used by gopls.
 	LicensesText string
 
-	// GoDiff is used in gopls/hooks to get Myers' diff
-	GoDiff bool
-
 	// Whether staticcheck is supported.
 	StaticcheckSupported bool
 
-	// ComputeEdits is used to compute edits between file versions.
-	ComputeEdits DiffFunction
-
 	// URLRegexp is used to find potential URLs in comments/strings.
 	//
 	// Not all matches are shown to the user: if the matched URL is not detected
@@ -525,12 +513,6 @@ type InternalOptions struct {
 	// This option applies only during initialization.
 	ShowBugReports bool
 
-	// NewDiff controls the choice of the new diff implementation. It can be
-	// 'new', 'old', or 'both', which is the default. 'both' computes diffs with
-	// both algorithms, checks that the new algorithm has worked, and write some
-	// summary statistics to a file in os.TmpDir().
-	NewDiff string
-
 	// SubdirWatchPatterns configures the file watching glob patterns registered
 	// by gopls.
 	//
@@ -661,16 +643,6 @@ const (
 	Structured HoverKind = "Structured"
 )
 
-type MemoryMode string
-
-const (
-	ModeNormal MemoryMode = "Normal"
-	// In DegradeClosed mode, `gopls` will collect less information about
-	// packages without open files. As a result, features like Find
-	// References and Rename will miss results in such packages.
-	ModeDegradeClosed MemoryMode = "DegradeClosed"
-)
-
 type VulncheckMode string
 
 const (
@@ -698,15 +670,15 @@ type OptionResults []OptionResult
 
 type OptionResult struct {
 	Name  string
-	Value interface{}
+	Value any
 	Error error
 }
 
-func SetOptions(options *Options, opts interface{}) OptionResults {
+func SetOptions(options *Options, opts any) OptionResults {
 	var results OptionResults
 	switch opts := opts.(type) {
 	case nil:
-	case map[string]interface{}:
+	case map[string]any:
 		// If the user's settings contains "allExperiments", set that first,
 		// and then let them override individual settings independently.
 		var enableExperiments bool
@@ -734,7 +706,7 @@ func SetOptions(options *Options, opts interface{}) OptionResults {
 	return results
 }
 
-func (o *Options) ForClientCapabilities(clientName *protocol.Msg_XInitializeParams_clientInfo, caps protocol.ClientCapabilities) {
+func (o *Options) ForClientCapabilities(clientName *protocol.ClientInfo, caps protocol.ClientCapabilities) {
 	o.ClientInfo = clientName
 	// Check if the client supports snippets in completion items.
 	if caps.Workspace.WorkspaceEdit != nil {
@@ -770,7 +742,8 @@ func (o *Options) ForClientCapabilities(clientName *protocol.Msg_XInitializePara
 	// Check if the client supports diagnostic related information.
 	o.RelatedInformationSupported = caps.TextDocument.PublishDiagnostics.RelatedInformation
 	// Check if the client completion support includes tags (preferred) or deprecation
-	if caps.TextDocument.Completion.CompletionItem.TagSupport.ValueSet != nil {
+	if caps.TextDocument.Completion.CompletionItem.TagSupport != nil &&
+		caps.TextDocument.Completion.CompletionItem.TagSupport.ValueSet != nil {
 		o.CompletionTags = true
 	} else if caps.TextDocument.Completion.CompletionItem.DeprecatedSupport {
 		o.CompletionDeprecated = true
@@ -784,9 +757,7 @@ func (o *Options) Clone() *Options {
 		ClientOptions:   o.ClientOptions,
 		InternalOptions: o.InternalOptions,
 		Hooks: Hooks{
-			GoDiff:               o.GoDiff,
 			StaticcheckSupported: o.StaticcheckSupported,
-			ComputeEdits:         o.ComputeEdits,
 			GofumptFormat:        o.GofumptFormat,
 			URLRegexp:            o.URLRegexp,
 		},
@@ -940,12 +911,7 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 		o.DirectoryFilters = filters
 
 	case "memoryMode":
-		if s, ok := result.asOneOf(
-			string(ModeNormal),
-			string(ModeDegradeClosed),
-		); ok {
-			o.MemoryMode = MemoryMode(s)
-		}
+		result.deprecated("")
 	case "completionDocumentation":
 		result.setBool(&o.CompletionDocumentation)
 	case "usePlaceholders":
@@ -1094,7 +1060,9 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 		result.setBool(&o.NoSemanticNumber)
 
 	case "expandWorkspaceToModule":
-		result.softErrorf("gopls setting \"expandWorkspaceToModule\" is deprecated.\nPlease comment on https://go.dev/issue/63536 if this impacts your workflow.")
+		// See golang/go#63536: we can consider deprecating
+		// expandWorkspaceToModule, but probably need to change the default
+		// behavior in that case to *not* expand to the module.
 		result.setBool(&o.ExpandWorkspaceToModule)
 
 	case "experimentalPostfixCompletions":
@@ -1161,7 +1129,7 @@ func (o *Options) set(name string, value interface{}, seen map[string]struct{})
 		// processed, so do nothing here.
 
 	case "newDiff":
-		result.setString(&o.NewDiff)
+		result.deprecated("")
 
 	case "subdirWatchPatterns":
 		if s, ok := result.asOneOf(
